@@ -47,7 +47,7 @@ st_as_sf.data.frame = function(x, ..., agr = NA_agr_, coords, wkt,
 		cc = as.data.frame(lapply(x[coords], as.numeric))
 		if (na.fail && any(is.na(cc)))
 			stop("missing values in coordinates not allowed")
-		classdim = getClassDim(rep(0, length(coords)), length(coords), dim, "POINT")
+		# classdim = getClassDim(rep(0, length(coords)), length(coords), dim, "POINT")
 		x$geometry = structure( points_rcpp(as.matrix(cc), dim),
 			n_empty = 0L, precision = 0, crs = NA_crs_,
 			bbox = structure(
@@ -452,7 +452,7 @@ as.data.frame.sf = function(x, ...) {
 
 #' @export
 #' @name st_geometry
-#' @details if \code{x} is of class \code{sf}, \code{st_drop_geometry} drops the geometry of its argument, and reclasses it accordingly; otherwise it does nothing.
+#' @details if \code{x} is of class \code{sf}, \code{st_drop_geometry} drops the geometry of its argument, and reclasses it accordingly; otherwise it returns \code{x} unmodified. 
 st_drop_geometry = function(x, ...) UseMethod("st_drop_geometry")
 
 #' @export
