@@ -6,7 +6,7 @@
 #'   values straddling 180 degrees. In general, using a projected
 #'   coordinate reference system is to be preferred, but this method permits a
 #'   geographical coordinate reference system to be used. This is the sf
-#'   equivalent of \code{\link[sp]{recenter}} in the sp package and
+#'   equivalent of \code{\link[sp:recenter-methods]{recenter}} in the sp package and
 #'   ST_ShiftLongitude in PostGIS.
 #'
 #' @param x object of class sf or sfc
@@ -35,7 +35,6 @@ st_shift_longitude = function(x) {
 st_shift_longitude.sfc = function(x, ...) {
 	xcrs = st_crs(x)
 	g = (x + c(360, 90)) %% c(360) - c(0, 90)
-	st_wrap_dateline(st_set_crs(g - c(180, 0), xcrs)) + c(180, 0)
 	st_set_crs(g, xcrs)
 }
 
