@@ -23,7 +23,7 @@
 
 #' @name st_as_sf
 #' @examples
-#' library(sp)
+#' if (require(sp, quietly = TRUE)) {
 #' x = rbind(c(-1,-1), c(1,-1), c(1,1), c(-1,1), c(-1,-1))
 #' x1 = 0.1 * x + 0.1
 #' x2 = 0.1 * x + 0.4
@@ -40,15 +40,9 @@
 #'    Polygon(y[5:1,]), Polygon(y1), Polygon(x1), Polygon(y3)), "ID1")
 #' p2 = Polygons(list( Polygon(z[5:1,]), Polygon(z2), Polygon(z3), Polygon(z1)),
 #'   "ID2")
-#' if (require("rgeos")) {
-#'   r = createSPComment(SpatialPolygons(list(p1,p2)))
-#'   comment(r)
-#'   comment(r@polygons[[1]])
-#'   scan(text = comment(r@polygons[[1]]), quiet = TRUE)
-#'   library(sf)
-#'   a = st_as_sf(r)
-#'   summary(a)
-#' }
+#' r = SpatialPolygons(list(p1,p2))
+#' a = suppressWarnings(st_as_sf(r))
+#' summary(a)
 #' demo(meuse, ask = FALSE, echo = FALSE)
 #' summary(st_as_sf(meuse))
 #' summary(st_as_sf(meuse.grid))
@@ -58,6 +52,7 @@
 #' pol.grd = as(meuse.grid, "SpatialPolygonsDataFrame")
 #' # summary(st_as_sf(pol.grd))
 #' # summary(st_as_sf(as(pol.grd, "SpatialLinesDataFrame")))
+#' }
 #' @export
 st_as_sf.Spatial = function(x, ...) {
 	if ("data" %in% slotNames(x))
@@ -142,12 +137,75 @@ st_as_sfc.SpatialLines = function(x, ..., precision = 0.0, forceMulti = FALSE) {
 
 #' @name st_as_sfc
 #' @export
+
 st_as_sfc.SpatialPolygons = function(x, ..., precision = 0.0, forceMulti = FALSE) {
 	lst = if (forceMulti || any(sapply(x@polygons, function(x) moreThanOneOuterRing(x@Polygons)))) {
 		if (is.null(comment(x)) || comment(x) == "FALSE") {
-			if (!requireNamespace("rgeos", quietly = TRUE))
-				stop("package rgeos required for finding out which hole belongs to which exterior ring")
-			x = rgeos::createSPComment(x)
+#			if (!requireNamespace("rgeos", quietly = TRUE))
+#				stop("package rgeos required for finding out which hole belongs to which exterior ring")
+#			x = rgeos::createSPComment(x)
+			# https://github.com/r-spatial/sf/pull/1869/files/7f1921c9acc1000b92a81b3a0aa7126330d4ef12..cfa303c8fcdd0b9a7ea33eae402c1135bb8e50ba :
+			# warning("no comment found showing which hole belongs to which exterior ring")
+			# (warning causes revdep problem in pkg amt)
+			process_pl_comment <- function(pl) {
+				ID <- slot(pl, "ID")
+				crds <- lapply(slot(pl, "Polygons"), function(xx) slot(xx, "coords"))
+				holes <- sapply(slot(pl, "Polygons"), slot, "hole")
+				raw0 <- st_sfc(lapply(crds, function(x) st_polygon(list(x))))
+				if (!any(holes)) {
+					val <- st_union(st_make_valid(raw0))
+				} else {
+					wkts <- vector("list", sum(!holes))
+					for (i in seq_along(wkts)) {
+						wkts[[i]] <- st_as_text(raw0[!holes][i])
+					}
+					cp0 <- st_contains(raw0[!holes], raw0[holes])
+					areas <- sapply(slot(pl, "Polygons"), slot, "area")
+					hole_assigned <- rep(FALSE, sum(holes))
+					names(cp0) <- 1:length(cp0)
+                                        cp1 <- cp0[order(areas[!holes])]
+					for (i in seq_along(cp1)) {
+                                          cp1i <- cp1[[i]]
+					  tgt <- as.integer(names(cp1[i]))
+                                          if (length(cp1i) > 0L) {
+                                            for (j in cp1i) {
+					      wkts[[tgt]] <- paste(sub("))", "),", wkts[[tgt]]), sub("POLYGON \\(", "", st_as_text(raw0[holes][j])))
+					      hole_assigned[j] <- TRUE
+					    }
+                                          }
+					  for (ii in i:length(cp1)) {
+                                            for (j in cp1i) {
+					      cp1[[ii]] <- setdiff(cp1[[ii]], j)
+					    }
+					  }
+					}
+					if (any(!hole_assigned))
+					  warning("orphaned hole, cannot find containing polygon")
+					raw0 <- st_as_sfc(wkts)
+                                        raw1 <- st_make_valid(raw0)
+                                        if (any(st_is(raw1, "GEOMETRYCOLLECTION"))) 
+                                          raw1 <- st_collection_extract(raw1, "POLYGON")
+					val <- st_union(raw1)
+				}
+				if (inherits(val, "sfc_GEOMETRYCOLLECTION"))
+					val = st_collection_extract(val, "POLYGON")
+				res <- slot(as(val, "Spatial"), "polygons")[[1]]
+				slot(res, "ID") <- ID
+				res
+			}
+#			process_pl_comment <- function(pl) {
+#				ID <- slot(pl, "ID")
+#				crds <- lapply(slot(pl, "Polygons"), function(xx) slot(xx, "coords"))
+#				raw <- st_sfc(st_polygon(crds))
+#				val <- st_make_valid(raw)
+#				if (inherits(val, "sfc_GEOMETRYCOLLECTION"))
+#					val = st_collection_extract(val, "POLYGON")
+#				res <- slot(as(val, "Spatial"), "polygons")[[1]]
+#				slot(res, "ID") <- ID
+#				res
+#			}
+			slot(x, "polygons") <- lapply(slot(x, "polygons"), process_pl_comment)
+			comment(x) <- "TRUE"
 		}
 		lapply(x@polygons, function(y)
 			st_multipolygon(Polygons2MULTIPOLYGON(y@Polygons, comment(y))))
@@ -228,6 +286,7 @@ setAs("XY", "Spatial", function(from) as(st_sfc(from), "Spatial"))
 #' @export
 #' @examples
 #' nc <- st_read(system.file("shape/nc.shp", package="sf"))
+#' if (require(sp, quietly = TRUE)) {
 #' # convert to SpatialPolygonsDataFrame
 #' spdf <- as_Spatial(nc)
 #' # identical to
@@ -236,6 +295,7 @@ setAs("XY", "Spatial", function(from) as(st_sfc(from), "Spatial"))
 #' as(st_geometry(nc), "Spatial")
 #' # back to sf
 #' as(spdf, "sf")
+#' }
 as_Spatial = function(from, cast = TRUE, IDs = paste0("ID", seq_along(from))) {
 	if (inherits(from, "sf")) {
 		geom = st_geometry(from)
@@ -338,16 +398,12 @@ setAs("crs", "CRS", function(from) CRS_from_crs(from))
 CRS_from_crs = function(from) {
 	if (! requireNamespace("sp", quietly = TRUE))
 		stop("package sp required, please install it first")
-	if (is.na(from))
-		sp::CRS(NA_character_)
-	else if (CPL_proj_version() >= "6.0.0" && CPL_gdal_version() >= "3.0.0") {
-		# we don't use sp::CRS(SRS_string = from$wkt) as rgdal may not be available,
-		# which would break, and from$wkt has already been validated by GDAL:
-		nm <- "CRS"
-		attr(nm, "package") <- "sp" # See ?new:
-		obj <- new(nm, projargs = from$proj4string)
+	nm <- "CRS"
+	attr(nm, "package") <- "sp" # See ?new:
+	obj <- new(nm, projargs = from$proj4string)
+	if (!is.na(from$wkt) && CPL_proj_version() >= "6.0.0" && CPL_gdal_version() >= "3.0.0")
 		comment(obj) <- from$wkt
-		obj
-	} else
-		sp::CRS(from$proj4string)
+	obj
+	# we don't use sp::CRS(SRS_string = from$wkt) as rgdal may not be available,
+	# which would break, and from$wkt has already been validated by GDAL:
 }
