@@ -25,7 +25,11 @@ dplyr_reconstruct.sf = function(data, template) {
 
 group_split.sf <- function(.tbl, ..., .keep = TRUE) {
 	 class(.tbl) = setdiff(class(.tbl), "sf")
-     lapply(dplyr::group_split(.tbl, ..., .keep = .keep), st_as_sf)
+	 if (inherits(.tbl, "rowwise_df")) {
+	 	lapply(dplyr::group_split(.tbl, ...), st_as_sf)
+	 } else {
+	 	lapply(dplyr::group_split(.tbl, ..., .keep = .keep), st_as_sf)	
+	 }
 }
 
 #' Tidyverse methods for sf objects (remove .sf suffix!)
@@ -567,10 +571,10 @@ unnest.sf = function(data, ..., .preserve = NULL) {
 #' @details see \link[pillar]{type_sum}
 type_sum.sfc <- function(x, ...) {
 	cls = substring(class(x)[1], 5)
-	if (is.na(st_is_longlat(x)))
-		cls
-	else
-		paste0(cls, " [", enc2utf8(as.character(units(st_crs(x, parameters = TRUE)$ud_unit))), "]")
+	u = st_crs(x)$ud_unit
+	if (!is.null(u)) # add [units]:
+		cls = paste0(cls, " [", enc2utf8(as.character(units(u))), "]")
+	cls
 }
 
 #' Summarize simple feature item for tibble
@@ -592,6 +596,13 @@ pillar_shaft.sfc <- function(x, ...) {
 	pillar::new_pillar_shaft_simple(out, align = "right", min_width = 25)
 }
 
+#' @name tidyverse
+drop_na.sf <- function(x, ...) {
+	sf_column_name = attr(x, "sf_column")
+	class(x) <- setdiff(class(x), "sf")
+	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
+}
+
 #nocov start
 register_all_s3_methods = function() {
 	has_dplyr_1.0 =
@@ -622,6 +633,7 @@ register_all_s3_methods = function() {
 	register_s3_method("dplyr", "summarise", "sf")
 	register_s3_method("dplyr", "transmute", "sf")
 	register_s3_method("dplyr", "ungroup", "sf")
+	register_s3_method("tidyr", "drop_na", "sf")
 	register_s3_method("tidyr", "gather", "sf")
 	register_s3_method("tidyr", "pivot_longer", "sf")
 	register_s3_method("tidyr", "pivot_wider", "sf")
