@@ -29,8 +29,12 @@
 #  endif
 # endif
 # if GEOS_VERSION_MINOR >= 10
+#  define HAVE310
 #  define HAVE3101
 # endif
+# if GEOS_VERSION_MINOR >= 11
+#  define HAVE311
+# endif
 #else
 # if GEOS_VERSION_MAJOR > 3
 #  define HAVE340
@@ -39,7 +43,9 @@
 #  define HAVE361
 #  define HAVE380
 #  define HAVE390
+#  define HAVE310
 #  define HAVE3101
+#  define HAVE311
 # endif
 #endif
 
@@ -199,7 +205,7 @@ std::vector<GeomPtr> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::Li
 		}
 #ifdef HAVE_390
 		else if (set_precision)
-			g[i] = geos_ptr(GEOSGeom_setPrecision_r(hGEOSCtxt, g[i].get(), precision, 0), hGEOSCtxt);
+			g[i] = geos_ptr(GEOSGeom_setPrecision_r(hGEOSCtxt, g[i].get(), precision, GEOS_PREC_VALID_OUTPUT), hGEOSCtxt);
 #endif
 	}
 	GEOSWKBReader_destroy_r(hGEOSCtxt, wkb_reader);
@@ -797,6 +803,15 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 	} else if (op == "boundary") {
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = geos_ptr(chkNULL(GEOSBoundary_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
+	} else if (op == "concave_hull") {
+#ifdef HAVE311
+		double ratio = bufferDist[0];
+		unsigned int allowHoles = preserveTopology[0];
+		for (size_t i = 0; i < g.size(); i++)
+			out[i] = geos_ptr(chkNULL(GEOSConcaveHull_r(hGEOSCtxt, g[i].get(), ratio, allowHoles)), hGEOSCtxt);
+#else
+		Rcpp::stop("st_concave_hull requires GEOS >= 3.11");
+#endif
 	} else if (op == "convex_hull") {
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = geos_ptr(chkNULL(GEOSConvexHull_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
@@ -836,6 +851,13 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 				dTolerance[i], bOnlyEdges)), hGEOSCtxt);
 	} else
 #endif
+#ifdef HAVE310
+	if (op == "triangulate_constrained") {
+		for (size_t i = 0; i < g.size(); i++)
+			out[i] = geos_ptr(chkNULL(GEOSConstrainedDelaunayTriangulation_r(hGEOSCtxt, g[i].get())), 
+							hGEOSCtxt);
+	} else
+#endif
 #ifdef HAVE370
 	if (op == "reverse") {
 		for (size_t i = 0; i < g.size(); i++) {
